from __future__ import unicode_literals
from django import forms
from django.db import models

from .widgets import AnyLinkAddOrChangeWidget


class AnyLinkAddOrChangeField(forms.ModelChoiceField):
    def __init__(self, *args, **kwargs):
        kwargs['widget'] = AnyLinkAddOrChangeWidget(kwargs.pop('rel'))
        super(AnyLinkAddOrChangeField, self).__init__(*args, **kwargs)


class AnyLinkField(models.ForeignKey):
    def __init__(self, to='anylink.AnyLink', **kwargs):
        kwargs['verbose_name'] = kwargs.pop('verbose_name', 'Link')
        super(AnyLinkField, self).__init__(to, **kwargs)

    def formfield(self, **kwargs):
        defaults = {
            'form_class': AnyLinkAddOrChangeField,
            'rel': self.rel,
            'to_field_name': self.rel.field_name,
        }
        defaults.update(kwargs)
        return super(AnyLinkField, self).formfield(**defaults)



from __future__ import unicode_literals
from django import forms, VERSION
from django.conf import settings
from django.contrib import admin
from django.core.urlresolvers import reverse
from django.utils.html import escape
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _


ICON_FILENAME = 'icon_deletelink.gif'
if VERSION[:2] >= (1, 9):
    ICON_FILENAME = 'icon-deletelink.svg'

CHANGE_LINK = (
    u'<a href="{0}{1}" class="show-popup" id="lookup_id_{2}" '
    u'onclick="return window.AnyLinkAddOrChangeWidget.show(this);" '
    u'data-add="{3}" data-change="{4}">{5}</a>'
)

SELECT_LINK = (
    u'&nbsp;<a href="{0}{1}" class="show-popup" id="lookup_id_{2}" '
    u'onclick="return window.AnyLinkAddOrChangeWidget.select(this);">{3}</a>'
)

DELETE_IMG = (
    u'&nbsp;<img src="{0}admin/img/' + ICON_FILENAME + u'" id="delete_id_{1}" '
    u'onclick="return window.AnyLinkAddOrChangeWidget.delete(this);" '
    u'style="cursor:pointer{2}" />'
)


class AnyLinkAddOrChangeWidget(forms.TextInput):
    input_type = 'hidden'

    # We need to pretend that our widget is a non-hidden input to ensure Django
    # doesn't remove the form-row.
    is_hidden = False

    class Media:
        js = ('anylink/anylink-addorchangewidget.js',)

    def __init__(self, rel, admin_site=None, attrs=None, using=None):
        self.rel = rel
        self.admin_site = admin_site or admin.site
        self.db = using
        super(AnyLinkAddOrChangeWidget, self).__init__(attrs)

    def render(self, name, value, attrs=None):
        if attrs is None:
            attrs = {}

        output = [super(AnyLinkAddOrChangeWidget, self).render(name, value, attrs)]
        output.append(self.label_for_value(name, value))

        if self.rel.to in self.admin_site._registry:
            related_url = reverse('admin:{0}_{1}_changelist'.format(
                self.rel.to._meta.app_label, self.rel.to._meta.model_name
            ), current_app=self.admin_site.name)

            params = self.url_parameters()
            if params:
                url_params = '?' + '&amp;'.join(
                    ['%s=%s' % (k, v) for k, v in list(params.items())])
            else:
                url_params = ''

            output.append(CHANGE_LINK.format(
                related_url, url_params, name,
                _('Add link'), _('Change link'),
                not value and _('Add link') or _('Change link')
            ))

            if getattr(settings, 'ANYLINK_ALLOW_MULTIPLE_USE', False):
                output.append(SELECT_LINK.format(
                    related_url, url_params, name,
                    _('Select link'),
                ))

        output.append(self.delete_button(name, value))
        return mark_safe(''.join(output))

    def url_parameters(self):
        from django.contrib.admin.views.main import TO_FIELD_VAR
        return {TO_FIELD_VAR: self.rel.get_related_field().name}

    def label_for_value(self, name, value):
        try:
            obj_repr = escape(
                self.rel.to._default_manager.using(self.db).get(
                    **{self.rel.get_related_field().name: value}
                )
            )
        except (ValueError, self.rel.to.DoesNotExist):
            obj_repr = ''

        return u'<strong id="name_id_{0}">{1}</strong>&nbsp;'.format(name, obj_repr)

    def delete_button(self, name, value):
        return DELETE_IMG.format(
            settings.STATIC_URL, name, ';display:none' if not value else '')



from django import forms
from django.conf import settings
from django.utils.encoding import force_text
from django.utils.translation import ugettext
from django.utils.translation import ugettext_lazy as _

from .models import AnyLink


class AnyLinkAdminForm(forms.ModelForm):

    confirmation = forms.BooleanField(
        label=_('Confirmation'), widget=forms.HiddenInput(), required=False)

    class Meta:
        model = AnyLink
        fields = '__all__'

    def clean(self):
        data = self.cleaned_data

        if self.instance.pk and getattr(settings, 'ANYLINK_ALLOW_MULTIPLE_USE', False):
            objects = self.instance.get_used_by()
            if len(objects) > 1 and not data.get('confirmation'):
                self.fields['confirmation'].widget = forms.CheckboxInput()
                self.fields['confirmation'].required = True
                self.fields['confirmation'].label = ugettext('Please confirm your changes.')

                objects_used = u', '.join([force_text(obj) for obj in objects])
                msg = ugettext(
                    u'The following objects are using this link already: {0}').format(
                        objects_used)
                raise forms.ValidationError(msg)
        return data



from __future__ import unicode_literals
from django.core.exceptions import ImproperlyConfigured, ValidationError
from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.utils.encoding import python_2_unicode_compatible

from .compat import get_model


@python_2_unicode_compatible
class BaseLink(object):
    name = None
    verbose_name = None
    provided_fields = None

    def __init__(self, **kwargs):
        self.kwargs = kwargs

    def get_name(self):
        return self.kwargs.get(
            'name', self.name or self.__class__.__name__.lower())

    def get_verbose_name(self):
        retval = self.kwargs.get('verbose_name', None)
        if retval is None and self.verbose_name is not None:
            retval = self.verbose_name
        return retval or self.__class__.__name__

    def __str__(self):
        return self.get_verbose_name()

    def configure_model(self, model):
        pass

    def configure_modeladmin(self, modeladmin):
        pass

    def get_absolute_url(self, obj):
        raise NotImplementedError('`get_absolute_url` is not implemented yet')

    def get_provided_fields(self):
        return self.provided_fields or (self.get_name(),)

    def clean(self, link):
        if not getattr(link, self.get_name()):
            raise ValidationError(_('{0} is required').format(self.get_verbose_name()))


class ExternalLink(BaseLink):
    name = 'external_url'
    verbose_name = _('external url')
    provided_fields = ('external_url',)

    def configure_model(self, model):
        model.add_to_class(
            self.get_name(),
            models.URLField(self.get_verbose_name(), max_length=255, blank=True)
        )

    def get_absolute_url(self, link):
        return getattr(link, self.get_name()) or None


class ModelLink(BaseLink):
    def __init__(self, **kwargs):
        super(ModelLink, self).__init__(**kwargs)

        if 'model' not in self.kwargs:
            raise ImproperlyConfigured('Please provide a model path')

        self.model = get_model(*self.kwargs['model'].split('.', 1))

        if not hasattr(self.model, 'get_absolute_url'):
            raise ImproperlyConfigured(
                '{0} does not implement `get_absolute_url`'.format(
                    self.model.__name__))

    def get_name(self):
        return self.kwargs.get('name', self.model.__name__.lower())

    def get_verbose_name(self):
        return self.kwargs.get('verbose_name', self.model._meta.verbose_name)

    def configure_model(self, model):
        model.add_to_class(self.get_name(), models.ForeignKey(
            self.model, blank=True, null=True, verbose_name=self.get_verbose_name()))

    def configure_modeladmin(self, modeladmin):
        modeladmin.raw_id_fields = list(modeladmin.raw_id_fields) + [self.get_name()]

    def get_absolute_url(self, link):
        obj = getattr(link, self.get_name())
        return obj and str(obj.get_absolute_url()) or None



import django
from django.conf import settings


def get_model(app_label, name):
    if django.VERSION < (1, 7):
        from django.db import models
        model = models.get_model(app_label, name)
    else:
        from django.apps import apps
        model = apps.get_registered_model(app_label, name)
    return model


def is_installed(label):
    # Configure django modeladmin
    if django.VERSION[:2] >= (1, 7):
        from django.apps import apps
        return apps.is_installed(label)
    return label in settings.INSTALLED_APPS


def get_app_module(module_name):
    if django.VERSION[:2] >= (1, 7):
        from django.apps import apps
        return apps.get_app_config(module_name.split('.')[-1]).module
    from django.utils.importlib import import_module
    return import_module(module_name)


def get_all_related_objects(cls):
    if django.VERSION[:2] <= (1, 7):
        # Due to some circumstances, it might happen that the related objects
        # cache is filled before all models are loaded. At this place, we reset
        # the cache if the list is not filled with at least one entry.
        related_objects = cls._meta.get_all_related_objects()

        if len(related_objects) < 1:
            cls._meta._fill_related_objects_cache()
            related_objects = cls._meta.get_all_related_objects()

        return related_objects
    else:
        return [
            f for f in cls._meta.get_fields()
            if f.one_to_many and f.auto_created
        ]


def add_error(form, field, msg, cleaned_data):
    if django.VERSION[:2] <= (1, 6):
        form._errors[field] = form.error_class([msg])
        cleaned_data.pop(field, None)
    else:
        form.add_error(field, msg)



# This model is for Django 1.7 compatibility. It should not be imported anywhere.
from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class AnyLinkConfig(AppConfig):

    name = 'anylink'
    verbose_name = _("AnyLink")

    def ready(self):
        from anylink.models import do_anylink_extension_setup

        for model in self.get_models():
            do_anylink_extension_setup(model)



from __future__ import unicode_literals
import re

import django
from django.contrib import admin
from django.conf import settings
from django.template.response import SimpleTemplateResponse
from django.utils.html import escape
try:
    import importlib
except ImportError:
    from django.utils import importlib

from .forms import AnyLinkAdminForm
from .models import AnyLink


EDITOR_ID_RE = re.compile('^[\w\-]+$')


class AnyLinkAdmin(admin.ModelAdmin):
    # TODO: Add test for `get_absolute_url` for py3k
    list_display = ('get_absolute_url', 'link_type', 'text')
    list_filter = ('link_type', 'target')
    search_fields = ('text', 'title')
    form = AnyLinkAdminForm

    def __init__(self, *args, **kwargs):
        super(AnyLinkAdmin, self).__init__(*args, **kwargs)
        if django.VERSION < (1, 7):
            for extension in list(self.model.extensions.values()):
                extension.configure_modeladmin(self)

    def get_form(self, request, obj=None, **kwargs):
        defaults = {}
        custom_form = getattr(settings, 'ANYLINK_ADMIN_FORM', None)

        if custom_form is not None:
            module, form_class = custom_form.rsplit('.', 1)
            defaults.update({
                'form': getattr(importlib.import_module(module), form_class),
            })

        defaults.update(kwargs)
        return super(AnyLinkAdmin, self).get_form(request, obj, **defaults)

    def get_model_perms(self, request):
        # Adding is disabled to hide the add buttons.
        return {
            'add': False,
            'change': self.has_change_permission(request),
            'delete': self.has_delete_permission(request),
        }

    def render_change_form(self, request, context, *args, **kwargs):
        context.update({
            'link_extensions': list(self.model.extensions.values())
        })

        return super(AnyLinkAdmin, self).render_change_form(
            request, context, *args, **kwargs)

    def response_add(self, request, obj):
        if self.is_addorchange_popup(request):
            return self.response_addorchange(request, obj)

        if self.is_rtelink_popup(request):
            return self.response_rtelink(request, obj)

        return super(AnyLinkAdmin, self).response_add(request, obj)

    def response_change(self, request, obj):
        if self.is_addorchange_popup(request):
            return self.response_addorchange(request, obj)

        if self.is_rtelink_popup(request):
            return self.response_rtelink(request, obj)

        return super(AnyLinkAdmin, self).response_change(request, obj)

    def is_rtelink_popup(self, request):
        return (
            '_popup' in request.POST and 'ed' in request.GET and
            EDITOR_ID_RE.match(request.GET['ed']) is not None
        )

    def response_rtelink(self, request, obj):
        return SimpleTemplateResponse(
            'admin/anylink/anylink/rtelink_response.html', {
                'editor_id': request.GET['ed'],
                'link_id': obj.get_rtelink_id()
            })

    def is_addorchange_popup(self, request):
        return ('_popup' in request.POST and 'aoc' in request.GET)

    def response_addorchange(self, request, obj):
        return SimpleTemplateResponse(
            'admin/anylink/anylink/addorchange_response.html', {
                'link_id': obj.pk,
                'link_name': escape(str(obj))
            })

admin.site.register(AnyLink, AnyLinkAdmin)
